
#ifndef Y_RGBABUFFER_H
#define Y_RGBABUFFER_H

struct RGBABuffer;

#include <Y/y.h>
#include <Y/util/rectangle.h>
#include <Y/screen/renderer.h>
#include <stdint.h>

/* Implementation of a buffer that stores Red-Green-Blue-Alpha data. */

struct RGBABuffer * rgbabufferCreate    ();
void                rgbabufferDestroy   (struct RGBABuffer *);

void rgbabufferSetSize     (struct RGBABuffer *, int w, int h);
void rgbabufferGetSize     (const struct RGBABuffer *, int *wp, int *hp);

void     rgbabufferSetPixel (struct RGBABuffer *, int x, int y, uint32_t col);
uint32_t rgbabufferGetPixel (const struct RGBABuffer *, int x, int y);

/* Obtain a Painter for this buffer.
 *
 * Returns a painter reference for this buffer.
 */
struct Painter *  rgbabufferGetPainter  (struct RGBABuffer *);

/* Render this buffer to the given renderer.
 * 
 * Only the portion of the buffer within RECT (which is in device
 * independent, buffer-relative  co-ordinates) will be renderered.
 * It will be rendered to the device-independent co-ordinates (X,Y).
 */
void  rgbabufferRender  (const struct RGBABuffer *, struct Renderer *,
                         int x, int y);

#endif

/* arch-tag: d687c025-5036-437c-915d-98bf15d84784
 */
