#ifndef Y_MODULES_THEME_INTERFACE_H
#define Y_MODULES_THEME_INTERFACE_H

struct Theme;

#include <Y/modules/module.h>
#include <Y/buffer/painter.h>
#include <Y/widget/widget.h>
#include <Y/widget/window.h>

void themeAdd    (struct Theme *);
void themeRemove (struct Theme *);

/* Themes are stacked, so themes that do not implement certain functions
 * will fall through to the next theme on the stack.
 *
 * The "basic" theme should always be loaded at the bottom of the stack to
 * handle the final fall through case.
 */

struct Theme
{
  char *name;
  struct Theme *nextTheme;    /* FOR Y USE ONLY: DO NOT USE. */

  void (*drawBackgroundPane)  (struct Painter *, int x, int y, int w, int h);

  void (*drawButtonPane)      (struct Painter *, int x, int y, int w, int h,
                               enum WidgetState state);

  struct Font *
       (*getDefaultFont)      ();

  /* windows are almost entirely handled by the theme, due to their
   * highly themic nature
   */
  void (*windowInit)          (struct Window *);
  void (*windowPaint)         (struct Window *, struct Painter *);
  int  (*windowGetRegion)     (struct Window *, int, int);
  int  (*windowPointerMotion) (struct Window *, int, int, int, int);
  int  (*windowPointerButton) (struct Window *, int, int, int, int);
  void (*windowReconfigure)   (struct Window *, int *, int *, int *, int *,
                               int *, int *);
  void (*windowResize)        (struct Window *);

};

#endif

/* arch-tag: eedf9fca-0711-4192-a4e2-c0c4cb61234b
 */
