#include <Y/screen/renderer.h>
#include <Y/screen/rendererclass.h>

#include <Y/util/yutil.h>

#include <stdlib.h>

struct RenderRegion
{
  struct Rectangle clip;
  int translateX, translateY;
};

void
rendererInitialise (struct Renderer *self)
{
  self -> regions = listCreate ();
}

void
rendererComplete (struct Renderer *self)
{
  if (self != NULL)
    self -> c -> complete (self);
}

void
rendererDestroy (struct Renderer *self)
{
  if (self != NULL)
    {
      listDestroy (self -> regions, yfree);
      self -> c -> destroy (self);
    }
}

int
rendererEnter (struct Renderer *self, const struct Rectangle *rect,
               int dx, int dy)
{
  struct RenderRegion *reg = ymalloc (sizeof (struct RenderRegion));
  struct RenderRegion *prev = listPeekFront (self -> regions);
  reg -> clip.x = rect -> x;
  reg -> clip.y = rect -> y;
  reg -> clip.w = rect -> w;
  reg -> clip.h = rect -> h;
  reg -> translateX = dx;
  reg -> translateY = dy;
  if (prev != NULL)
    {
      reg -> clip.x += prev -> translateX;
      reg -> clip.y += prev -> translateY;
      reg -> translateX += prev -> translateX;
      reg -> translateY += prev -> translateY;
      if (!rectangleIntersect (&(reg -> clip), &(reg -> clip),
                               &(prev -> clip)))
         {
           yfree (reg);
           return 0;
         }
    }
  listPrepend (self -> regions, reg);
  return 1;
}

void
rendererLeave (struct Renderer *self)
{
  if (self != NULL)
    {
      if (listCount (self -> regions) != 0)
        {
          struct RenderRegion *r = listPopFront (self -> regions);
          yfree (r);
        }
    }
}

void
rendererBlitRGBAData (struct Renderer *self, int x, int y, uint32_t *data,
                      int w, int h, int s)
{
  struct Rectangle r = { x, y, w, h };
  struct Rectangle r2;
  if (self != NULL)
    {
      struct RenderRegion *reg = listPeekFront (self -> regions);
      if (reg != NULL)
        {
          r.x += reg -> translateX;
          r.y += reg -> translateY;
          if (!rectangleIntersect (&r2, &r, &(reg -> clip)))
            return;
        }
      self -> c -> blitRGBAData (self, r2.x, r2.y, 
                                 data + (r2.x - r.x) + s * (r2.y - r.y),
                                 r2.w, r2.h, s);
    }
}

void
rendererDrawFilledRectangle (struct Renderer *self, uint32_t colour,
                             int x, int y, int w, int h)
{
  struct Rectangle r = { x, y, w, h };
  if (self != NULL)
    {
      struct RenderRegion *reg = listPeekFront (self -> regions);
      if (reg != NULL)
        {
          r.x += reg -> translateX;
          r.y += reg -> translateY;
          rectangleIntersect (&r, &r, &(reg -> clip));
        }
      self -> c -> drawFilledRectangle (self, colour, r.x, r.y, r.w, r.h);
    }
}

/* arch-tag: 08567273-861d-4d25-b7b2-97ce1cd993dc
 */
