/* viewport.h */

#ifndef Y_SCREEN_VIEWPORT_H
#define Y_SCREEN_VIEWPORT_H

struct Viewport;

#include <Y/modules/videodriver_interface.h>
#include <Y/util/rectangle.h>
#include <Y/object/parameters.h>

/*
 * A Viewport is the concept of a view onto the abstract display.
 *
 * A viewport represents a rectangle which is held by the screen as one
 * of the objects that actually display things. A viewport knows its
 * video driver, which is the object that does the actual work.
 */

/* Create a new Viewport object that is referred to by DRIVER. */
struct Viewport  *viewportCreate (struct VideoDriver *driver);

/* Destroy a Viewport. */
void              viewportDestroy (struct Viewport *);

/* Invoke a special method on a viewport */
void              viewportSpecial (struct Viewport *, struct Parameters *);

/* Get the unique identifier for a viewport. */
int               viewportGetID (struct Viewport *);

/* Get the name of the video driver connected to the viewport. */
const char *      viewportGetName (struct Viewport *);

/* Request the resolutions that the viewport can handle. */
struct List *     viewportGetResolutions (struct Viewport *);
/* returns a list of struct VideoResolution */

/* Ask the viewport to set its resolution to a resolution. */
void              viewportSetResolution (struct Viewport *, const char *name);

/* Get the device independent co-ordinates of the Viewport. */
struct Rectangle *viewportGetRectangle (struct Viewport *);

/* Invalidate a region of the viewport. */
void              viewportInvalidateRectangle (struct Viewport *,
                                               const struct Rectangle *);

void              viewportSetSize (struct Viewport *, int, int);

/* Cause the viewport to update itself. */ 
void              viewportUpdate (struct Viewport *);

#endif

/* arch-tag: d1442762-99d7-47b6-9bfd-45d5b96a72b6
 */
