#include <Y/y.h>
#include <Y/util/list.h>
#include <Y/util/yutil.h>

struct ListNode
{
  void *data;
  struct ListNode *previous, *next;
};

struct List
{
  int count;
  struct ListNode *first, *last;
};

struct ListIterator
{
  struct List *list;
  struct ListNode *node;
};

struct List *
listCreate ()
{
  struct List *self = ymalloc (sizeof (struct List));
  self -> count = 0;
  self -> first = 0;
  self -> last = 0;
  return self;
}

void
listDestroy (struct List *self, void (*destructorFunction)(void *))
{
  struct ListNode *current = self -> first;
  while (current != NULL)
    {
      struct ListNode *next = current -> next;
      if (destructorFunction != NULL)
        destructorFunction (current -> data);
      yfree (current);
      current = next;
    }
  yfree (self);
}
 
int  
listCount (struct List *self)
{
  return self -> count;
}

void
listAppend (struct List *self, void *data)
{
  struct ListNode *newNode = ymalloc (sizeof (struct ListNode));
  newNode -> data = data;
  newNode -> next = NULL;
  newNode -> previous = self -> last;
  if (self -> last)
    {
      self -> last -> next = newNode;
      self -> last = newNode;
    }
  else
    {
      self -> first = newNode;
      self -> last = newNode;
    }
  self -> count++;
}

void 
listPrepend (struct List *self, void *data)
{
  struct ListNode *newNode = ymalloc (sizeof (struct ListNode));
  newNode -> data = data;
  newNode -> previous = NULL;
  newNode -> next = self -> first;
  if (self -> first)
    {
      self -> first -> previous = newNode;
      self -> first = newNode;
    }
  else
    {
      self -> first = newNode;
      self -> last = newNode;
    }
  self -> count++;
}

void *
listPeekFront (struct List *self)
{
  if (self -> first)
    return self -> first -> data;
  else
    return NULL;
}

void *
listPeekBack (struct List *self)
{
  if (self -> last)
    return self -> last -> data;
  else
    return NULL;
}

void *
listPopFront (struct List *self)
{
  struct ListNode *first = self -> first;
  void *data;
  if (first == NULL)
    return NULL;

  self -> first = first -> next;
  data = first -> data;
  if (self -> first == NULL)
    self -> last = NULL;
  yfree (first);
  self -> count --;
  return data;
}

void *
listPopBack (struct List *self)
{
  struct ListNode *last = self -> last;
  void *data;
  if (last == NULL)
    return NULL;

  self -> last = last -> previous;
  data = last -> data;
  if (self -> last == NULL)
    self -> first = NULL;
  yfree (last);
  self -> count --;
  return data;
}

struct ListIterator *
listGetStartIterator (struct List *self)
{
  struct ListIterator *iterator = ymalloc (sizeof (struct ListIterator));
  iterator -> list = self;
  iterator -> node = self -> first;
  return iterator;
}

struct ListIterator *
listGetEndIterator (struct List *self)
{
  struct ListIterator *iterator = ymalloc (sizeof (struct ListIterator));
  iterator -> list = self;
  iterator -> node = self -> last;
  return iterator;
}

void
listiteratorDestroy (struct ListIterator *self)
{
  yfree (self);
}

int listiteratorHasValue (struct ListIterator *self)
{
  return (self -> node != NULL) ? 1 : 0;
}

void *
listiteratorGet (struct ListIterator *self)
{
  if (self -> node == NULL)
    return NULL;
  else
    return self -> node -> data;
}

void *
listiteratorRemove (struct ListIterator *self)
{
  void * data;
  struct ListNode *node = self -> node;

  if (node == NULL)
    return NULL;

  if (node -> previous == NULL)
    self -> list -> first = node -> next;
  else
    node -> previous -> next = node -> next;

  if (node -> next == NULL)
    self -> list -> last = node -> previous;
  else
    node -> next -> previous = node -> previous;

  data = node -> data;
  self -> node = node -> next;
  yfree (node);
  return data;
}

void
listiteratorNext (struct ListIterator *self)
{
  if (self -> node == NULL)
    self -> node = self -> list -> first;
  else
    self -> node = self -> node -> next;
}

void
listiteratorPrevious (struct ListIterator *self)
{
  if (self -> node == NULL)
    self -> node = self -> list -> last;
  else
    self -> node = self -> node -> previous;
}

struct ListIterator *
listiteratorClone (struct ListIterator *self)
{
  struct ListIterator *iterator = ymalloc (sizeof (struct ListIterator));
  iterator -> list = self -> list;
  iterator -> node = self -> node;
  return iterator;
}

/* arch-tag: 2083da1c-23bc-4fc3-a9dd-00401b2c1fed
 */
