#ifndef Y_UTIL_LOG_H
#define Y_UTIL_LOG_H

#include <Y/y.h>
#include <stdio.h>
#include <syslog.h>

#if 1
#define Y_LOG_INIT()       do { openlog ("Y", LOG_CONS|LOG_NDELAY|LOG_PID, LOG_LOCAL7); } while (0)

#define Y_SILENT(str, ...)   
#define Y_DEBUG(str, ...)  do { syslog (LOG_DEBUG,   "%s:%d: " str "\n", __FUNCTION__, __LINE__, ##__VA_ARGS__ ); } while (0) 
#define Y_TRACE(str, ...)  do { syslog (LOG_INFO,    "%s:%d: " str "\n", __FUNCTION__, __LINE__, ##__VA_ARGS__ ); } while (0)
#define Y_INFO(str, ...)   do { syslog (LOG_INFO,              str "\n",                         ##__VA_ARGS__ ); } while (0)
#define Y_WARN(str, ...)   do { syslog (LOG_NOTICE,  "Warning: %s: "     str "\n", __FUNCTION__, ##__VA_ARGS__ ); } while (0)
#define Y_ERROR(str, ...)  do { syslog (LOG_WARNING, "Error: %s: "       str "\n", __FUNCTION__, ##__VA_ARGS__ ); } while (0)
#define Y_FATAL(str, ...)  do { syslog (LOG_ERR,     "Fatal Error: %s: " str "\n", __FUNCTION__, ##__VA_ARGS__ ); } while (0)

#else
#define Y_LOG_DEBUG 0x0010
#define Y_LOG_TRACE 0x0040
#define Y_LOG_WARN  0x0100
#define Y_LOG_ERROR 0x0400
#define Y_LOG_FATAL 0x1000

#define Y_LOG_LEVEL Y_LOG_TRACE

#define Y_LOG_INIT()

#define Y_LOG(lvl, string, ...) do { if (   lvl >= Y_LOG_LEVEL) printf("%s:%d: " string "\n", __FUNCTION__, __LINE__ ,##__VA_ARGS__ ); } while (0)
#define Y_SILENT(string, ...)
#define Y_DEBUG(string, ...) do { if (Y_LOG_DEBUG >= Y_LOG_LEVEL) printf("%s:%d: " string "\n", __FUNCTION__, __LINE__ ,##__VA_ARGS__ ); } while (0)
#define Y_TRACE(string, ...) do { if (Y_LOG_TRACE >= Y_LOG_LEVEL) printf("%s:%d: " string "\n", __FUNCTION__, __LINE__ ,##__VA_ARGS__ ); } while (0)
#define Y_WARN( string, ...) do { if (Y_LOG_WARN  >= Y_LOG_LEVEL) printf("%s:%d: " string "\n", __FUNCTION__, __LINE__ ,##__VA_ARGS__ ); } while (0)
#define Y_ERROR(string, ...) do { if (Y_LOG_ERROR >= Y_LOG_LEVEL) printf("%s:%d: " string "\n", __FUNCTION__, __LINE__ ,##__VA_ARGS__ ); } while (0)
#define Y_FATAL(string, ...) do { if (Y_LOG_FATAL >= Y_LOG_LEVEL) printf("%s:%d: " string "\n", __FUNCTION__, __LINE__ ,##__VA_ARGS__ ); abort (); } while (0)
#endif

#endif /* header guard */

/* arch-tag: 44a7ecc0-bdfa-434d-a47e-47fd7435fc9a
 */
