
#ifndef Y_WIDGET_WIDGET_H
#define Y_WIDGET_WIDGET_H

struct Widget;
struct WidgetTable;

enum WidgetState
{
  WIDGET_STATE_NORMAL,
  WIDGET_STATE_HOVER,
  WIDGET_STATE_FOCUS,
  WIDGET_STATE_SELECTED,
  WIDGET_STATE_PRESSED,
  WIDGET_STATE_CANCELLING,
  WIDGET_STATE_DISABLED
};

#include <Y/y.h>
#include <Y/const.h>
#include <Y/util/rectangle.h>
#include <Y/screen/renderer.h>
#include <Y/buffer/painter.h>
#include <Y/input/pointer.h>
#include <Y/input/keyboard.h>

struct Object *
       widgetToObject      (struct Widget *);

void   widgetGlobalToLocal (const struct Widget *, int *, int *);
void   widgetLocalToGlobal (const struct Widget *, int *, int *);

struct Rectangle *
       widgetGetRectangle  (const struct Widget *);

void   widgetGetPosition   (const struct Widget *, int *, int *);
void   widgetGetSize       (const struct Widget *, int *, int *);
void   widgetGetConstraints(const struct Widget *, int *, int *,
                            int *, int *, int *, int *);
int    widgetContainsPoint (const struct Widget *, int x, int y); 

struct Window *
       widgetGetWindow     (struct Widget *);

void   widgetMove          (struct Widget *, int, int);
void   widgetResize        (struct Widget *, int, int);
void   widgetReconfigure   (struct Widget *);

void   widgetSetContainer  (struct Widget *, struct Widget *);
void   widgetUnpack        (struct Widget *, struct Widget *);

void   widgetRender        (struct Widget *, struct Renderer *);
void   widgetPaint         (struct Widget *, struct Painter *);
void   widgetRepaint       (struct Widget *, struct Rectangle *);
void   widgetRerender      (struct Widget *, struct Rectangle *);

int    widgetPointerMotion (struct Widget *, int x, int y, int dx, int dy);
int    widgetPointerButton (struct Widget *, int x, int y, int b, int pressed);
void   widgetPointerEnter  (struct Widget *, int x, int y);
void   widgetPointerLeave  (struct Widget *);
int    widgetKeyboardRaw   (struct Widget *, enum YKeyCode, int pressed,
                                             int modifierState);

#endif /* header guard */

/* arch-tag: 1b0c27f4-f65e-48fa-a762-c2bac42cffc9
 */
