
#include "libY.h"
#include "internal.h"

#include <stdlib.h>
#include <sys/poll.h>
#include <unistd.h>
#include <stdio.h>

/* compile this file in if you want multi-fd access using select (); */

struct FDEntry
{
  int fd;
  int mask;
  void *data;
  void (*callback)(int, int, void*);
  struct FDEntry *next;
};

static struct FDEntry *entryList = NULL;
static int entryListCount = 0;

void
libY_initialise ()
{
  
}

void
libY_finalise ()
{

}

void
yRegisterFD (int fd, int mask, void *data, void (*call)(int, int, void*))
{
  /* check the fd is not already registered: */
  struct FDEntry *cur = entryList;
  while (cur != NULL)
    {
      if (cur -> fd == fd)
        return;
      cur = cur -> next;
    }

  cur = malloc (sizeof (struct FDEntry));
  if (cur == NULL)
    return;
  cur -> fd = fd;
  cur -> mask = mask;
  cur -> data = data;
  cur -> callback = call;
  cur -> next = entryList;
  entryList = cur;
  entryListCount++;
}

void
yChangeFD (int fd, int mask)
{
  struct FDEntry *cur = entryList;
  while (cur != NULL)
    {
      if (cur -> fd == fd)
        {
          cur -> mask = mask;
          return;
        }
      cur = cur -> next;
    }
}

void
yUnregisterFD (int fd)
{
  struct FDEntry *prev = entryList;
  struct FDEntry *cur = entryList;
  if (cur == NULL)
    return;
  if (cur -> fd == fd)
    {
      entryList = cur -> next;
    }
  else
    {
      cur = cur -> next;
      while (cur != NULL)
        {
          if (cur -> fd == fd)
            {
              prev -> next = cur -> next;
              break;
            }
          prev = cur;
          cur = cur -> next;
        }
    }
  if (cur != NULL)
    {
      free (cur);
      entryListCount--;
    }
}

void
libY_iterate (int timeout)
{
  struct pollfd fds[entryListCount];
  struct FDEntry *cur = entryList;
  int i = 0;
  int r;

  if (timeout < 0)
    timeout = 10000;

  while (cur != NULL)
    {
      fds[i].fd = cur -> fd;
      fds[i].events = 0;
      if ((cur -> mask & Y_LISTEN_READ) != 0)
        fds[i].events |= POLLIN;
      if ((cur -> mask & Y_LISTEN_WRITE) != 0)
        fds[i].events |= POLLOUT;
      i++;
      cur = cur -> next;
    }

  r = poll (fds, entryListCount, timeout);

  if (r > 0)
    {
      i = 0;
      cur = entryList;
      while (cur != NULL)
        {
          int mask = 0;
          if ((fds[i].revents & POLLIN) != 0)
            mask |= Y_LISTEN_READ;
          if ((fds[i].revents & POLLOUT) != 0)
            mask |= Y_LISTEN_WRITE;
          if (mask != 0)
            cur -> callback (cur -> fd, mask, cur -> data);
          i++;
          cur = cur -> next;
        }
    }

}


/* arch-tag: 310bc133-2f53-40c8-965f-0f51a0718acb
 */
