
#include "libY.h"
#include "internal.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

static int fd;

static void tcpSendMessage (struct Message *);
static int tcpRecvMessage (struct Message *, int);

static void tcpCallback (int fd, int mask, void *);

static struct Message incoming, outgoing;
static ssize_t sent, received;
static ssize_t sendLen, recvLen;

int
libY_tcpInitialise (const char *display)
{
  if (strncmp (display, "tcp:", 4) == 0)
    {
      int r;
      struct sockaddr_in sockaddr;
      struct hostent *host;
      char buffer[strlen (display)];
      char *portspec;

      sockaddr.sin_family = AF_INET;

      strcpy (buffer, display+4);
      portspec = strchr (buffer, ':');
      if (portspec == NULL)
        {
          sockaddr.sin_port = htons (8900);
        }
      else
        {
          *(portspec++) = '\0';
          sockaddr.sin_port = htons (strtoul (portspec, NULL, 10));
        }

      host = gethostbyname (buffer);
      if (host == NULL)
        {
          switch (h_errno)
            {
              case HOST_NOT_FOUND:
                fprintf (stderr, "Host not found.\n");
                return 0;
              case NO_ADDRESS:
                fprintf (stderr, "Couldn't find address for host.\n");
                return 0;
              default:
                fprintf (stderr, "Error resolving host name.\n");
                return 0;
            }
        }
      sockaddr.sin_addr.s_addr = *((in_addr_t *) host -> h_addr_list[0]);

      fd = socket (PF_INET, SOCK_STREAM, 0);
      r = connect (fd, (struct sockaddr *)&sockaddr, sizeof (sockaddr));

      if (r == 0)
        {
          libY_sendMessage = tcpSendMessage;
          libY_recvMessage = tcpRecvMessage;
          yRegisterFD (fd, 0, NULL, tcpCallback);
          return 1;
        }
      perror ("Failed to connect to Y Server via TCP");
    }
  return 0;
}

void
libY_tcpFinalise ()
{
  close (fd);
}

void
tcpSendMessage (struct Message *send)
{
  /* prepare the message to send */
  sent = -(MESSAGE_HEADER_LENGTH);
  sendLen = send -> len;

  outgoing.to   = (int) htonl (send -> to);
  outgoing.from = (int) htonl (send -> from);
  outgoing.cid  = (int) htonl (send -> cid);
  outgoing.oid  = (int) htonl (send -> oid);
  outgoing.op   = (int) htonl (send -> op);
  outgoing.meta = (int) htonl (send -> meta);
  outgoing.ext  = (int) htonl (send -> ext);
  outgoing.len  = (int) htonl (send -> len);
  outgoing.data = send -> data;

  while (sent < sendLen)
    {
      if (sent < 0)
        {
          ssize_t ret;
          struct iovec buffers[2];
          buffers[0].iov_base = ((uint8_t *)&outgoing) + MESSAGE_HEADER_LENGTH + sent;
          buffers[0].iov_len = -sent;
          buffers[1].iov_base = outgoing.data;
          buffers[1].iov_len = sendLen;
          ret = writev (fd, buffers, sendLen > 0 ? 2 : 1);
          if (ret < 1)
            {
              perror ("Writing (header) to Y Socket");
              abort ();
            }
          sent += ret;
        }
      else if (sent < sendLen)
        {
          ssize_t ret;
          ret = write (fd, outgoing.data + sent, sendLen - sent);
          if (ret < 1)
            {
              perror ("Writing (body) to Y Socket");
              abort ();
            }
          sent += ret;
        }
    }
}

int
tcpRecvMessage (struct Message *recv, int timeout)
{
  /* prepare the message to receive */
  struct timeval end, now;
  received = -(MESSAGE_HEADER_LENGTH);
  yChangeFD (fd, Y_LISTEN_READ);
  recvLen = 0;

  if (timeout >= 0)
    {
      gettimeofday (&now, NULL);
      end.tv_sec = (now.tv_sec + timeout / 1000);
      end.tv_usec = (now.tv_usec + (timeout % 1000) * 1000);
      while (end.tv_usec >= 1000000)
        {
          end.tv_sec ++;
          end.tv_usec -= 1000000;
        }
    }

  do
    {
      int remaining = -1;
      if (timeout >= 0 && received == -(MESSAGE_HEADER_LENGTH))
        remaining = (end.tv_sec - now.tv_sec) * 1000 +
                    (end.tv_usec - now.tv_usec) / 1000;
      libY_iterate (remaining);
      if (timeout >= 0 && received == -(MESSAGE_HEADER_LENGTH))
        {
          gettimeofday (&now, NULL);
          if (now.tv_sec > end.tv_sec ||
              (now.tv_sec == end.tv_sec && now.tv_usec >= end.tv_usec))
            {
              yChangeFD (fd, 0);
              return 0;
            }
        } 
    }
  while (received < recvLen);
  
  recv -> to   = (int) ntohl (incoming.to);
  recv -> from = (int) ntohl (incoming.from);
  recv -> cid  = (int) ntohl (incoming.cid);
  recv -> oid  = (int) ntohl (incoming.oid);
  recv -> op   = (int) ntohl (incoming.op);
  recv -> meta = (int) ntohl (incoming.meta);
  recv -> ext  = (int) ntohl (incoming.ext);
  recv -> len  = (int) ntohl (incoming.len);
  recv -> data = incoming.data;

  yChangeFD (fd, 0);
  return 1;
}

void
tcpCallback (int fd, int mask, void *v)
{
  if ((mask & Y_LISTEN_READ) != 0)
    {
      while (received < recvLen)
        {
          if (received < 0)
            {
              ssize_t ret;
              ret = read (fd,
               ((uint8_t*)&incoming) + MESSAGE_HEADER_LENGTH + received, -received);
              if (ret < 1)
                {
                  perror ("Reading (header) from Y Socket");
                  abort ();
                }
              received += ret;
              if (received == 0)
                {
                  recvLen = ntohl (incoming.len);
                  if (recvLen == 0)
                    incoming.data = NULL;
                  else
                    {
                      incoming.data = malloc (recvLen + 1);
                      incoming.data[recvLen] = '\0';
                    }
                }
            }
          else if (received < recvLen)
            {
              ssize_t ret;
              ret = read (fd, incoming.data + received, recvLen - received);
              if (ret < 1)
                {
                  perror ("Reading (body) from Y Socket");
                  abort ();
                }
              received += ret;
            }
        }
    }
}


/* arch-tag: fd8af59b-0a46-4c24-83f6-923345bbadd7
 */
